from django.core.cache import cache
from django.db.models.signals import post_save, post_delete

from tracking.models import UntrackedUserAgent, BannedIP

import logging

log = logging.getLogger('tracking.listeners')


def refresh_untracked_user_agents(sender, instance, created=False, **kwargs):
    """Update the cache of user agents that we do not track."""

    log.debug('Updating untracked user agents cache...')
    cache.set('_tracking_untracked_uas',
              UntrackedUserAgent.objects.all(),
              3600)


def refresh_banned_ips(sender, instance, created=False, **kwargs):
    """Update the cache of banned IP addresses."""

    log.debug('Updating banned IP cache...')
    cache.set('_tracking_banned_ips',
              [b.ip_address for b in BannedIP.objects.all()],
              3600)

post_save.connect(refresh_untracked_user_agents, sender=UntrackedUserAgent)
post_delete.connect(refresh_untracked_user_agents, sender=UntrackedUserAgent)

post_save.connect(refresh_banned_ips, sender=BannedIP)
post_delete.connect(refresh_banned_ips, sender=BannedIP)




from django.conf import settings
from django.conf import global_settings

TRACKING_BOGUS_IP = getattr(settings, 'TRACKING_BOGUS_IP', '10.10.10.10')
NO_TRACKING_PREFIXES = getattr(settings, 'NO_TRACKING_PREFIXES', [])

"""
Tracking timeout is set to 10 minutes by default.
"""
TRACKING_TIMEOUT = getattr(settings, 'TRACKING_TIMEOUT', '10')

"""
Tracking clean-up timeout is set to 24 hours by default.
"""
TRACKING_CLEANUP_TIMEOUT = getattr(settings, 'TRACKING_CLEANUP_TIMEOUT', 24)

DEFAULT_TRACKING_TEMPLATE = getattr(settings,
                                    'DEFAULT_TRACKING_TEMPLATE',
                                    'tracking/visitor_map.html')
USE_GEOIP = getattr(settings, 'TRACKING_USE_GEOIP', False)
CACHE_TYPE = getattr(settings, 'GEOIP_CACHE_TYPE', 4)

STATIC_URL = getattr(settings, 'STATIC_URL', global_settings.STATIC_URL)
MEDIA_URL = getattr(settings, 'MEDIA_URL', global_settings.MEDIA_URL)




from django.test import TestCase, Client, override_settings
from tracking.models import Visitor


def _create_session(client):
    """
    Add a session to a test client.

    In Django 1.7 it is impossible to do:

    s = self.client.session
    s['foo'] = 'bar'
    s.save()

    This bug is explained and fixed in Django ticket #21357.
    See https://github.com/django/django/commit/
    be88b062afaa58559bb12623e8ed8843f07b97a1
    """
    from importlib import import_module
    from django.conf import settings
    SessionStore = import_module(settings.SESSION_ENGINE).SessionStore
    s = SessionStore()
    s.save()
    client.cookies[settings.SESSION_COOKIE_NAME] = s.session_key
    return s


class VisitorMiddlewareTestCase(TestCase):

    def setUp(self):
        pass

    def test_ajax_call(self):
        """Ajax calls should be skipped."""
        client = Client()
        client.post('/ajax', HTTP_X_REQUESTED_WITH='XMLHttpRequest')
        self.assertEqual(Visitor.objects.count(), 0)

    def test_ip_address(self):
        """Test the ip address."""
        client = Client()
        client.post('/with_ip', REMOTE_ADDR='10.10.10.10')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].ip_address, '10.10.10.10')

    def test_user_agent(self):
        """Test the user agent."""
        client = Client()
        client.post('/with_user_agent', HTTP_USER_AGENT='Some User Agent')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].user_agent, 'Some User Agent')

    def test_without_session(self):
        """Test request without any session."""
        self.client = Client()
        self.client.post('/no_session', REMOTE_ADDR='10.10.10.10')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].session_key, '10.10.10.10:')

    def test_without_session_but_with_a_user_agent(self):
        """Test request without any session but with a username."""
        self.client = Client()
        self.client.post('/no_session_but_user_name',
                         HTTP_USER_AGENT='FooBar',
                         REMOTE_ADDR='10.10.10.10')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].session_key, '10.10.10.10:FooBar')

    def test_with_session(self):
        """Test request with a session instance."""
        self.client = Client()
        _create_session(self.client)
        session_key = self.client.session.session_key
        self.client.post('/hello', REMOTE_ADDR='10.10.10.10')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].session_key, session_key)

    def test_with_no_tracking_prefix(self):
        """
        Make sure request to a path starting with NO_TRACKING_PREFIXES are
        skipped.
        """
        self.client = Client()
        from tracking import settings
        # this is very bad, but since override_settings did not seem to work...
        settings.NO_TRACKING_PREFIXES = ['/fizz', '/fuzz']
        self.client.post('/fizz', REMOTE_ADDR='10.10.10.10')
        self.client.post('/fuzz', REMOTE_ADDR='11.11.11.11')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 0)

    def test_media_and_static_no_tracking_prefixes(self):
        """
        Make sure request to a path starting with a MEDIA_URL or STATIC_URL
        are skipped.
        """
        self.client = Client()
        from tracking import settings
        # this is very bad, but since override_settings did not seem to work...
        settings.ADMIN_URL = '/admin'
        settings.MEDIA_URL = '/media'
        self.client.post('/admin', REMOTE_ADDR='10.10.10.10')
        self.client.post('/media', REMOTE_ADDR='11.11.11.11')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 0)

    def test_referer(self):
        """Test refer(r)er."""
        self.client = Client()
        _create_session(self.client)
        self.client.post('/hello', HTTP_REFERER='http://elevenbits.com')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].referrer, 'http://elevenbits.com')

    def test_single_visitor_requests(self):
        """Test single visitor request."""
        self.client = Client()
        _create_session(self.client)
        self.client.post('/hello')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        self.assertEqual(all_visitors[0].url, '/hello')
        self.assertEqual(all_visitors[0].referrer, 'unknown')
        self.assertEqual(all_visitors[0].page_views, 1)

    def test_multiple_visitor_requests(self):
        """Test several visitor requests."""
        self.client = Client()
        # first user
        self.client.post('/hello', REMOTE_ADDR='10.10.10.10')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 1)
        session_start = all_visitors[0].session_start
        # second user
        self.client.post('/hello', REMOTE_ADDR='11.11.11.11')
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 2)
        # three new requests from first user
        self.client.post('/hello', REMOTE_ADDR='10.10.10.10')
        self.client.post('/hello', REMOTE_ADDR='10.10.10.10')
        self.client.post('/hello', REMOTE_ADDR='10.10.10.10')
        # assert
        all_visitors = Visitor.objects.all()
        self.assertEqual(len(all_visitors), 2)
        self.assertEqual(session_start, all_visitors[0].session_start)



from django.conf.urls import patterns, include, url

from tracking.views import update_active_users, get_active_users, display_map
# from tracking import settings

urlpatterns = patterns(
    '',
    url(r'^refresh/$', update_active_users, name='refresh'),
    url(r'^refresh/json/$', get_active_users, name='get'),
)

# if getattr(settings, 'TRACKING_USE_GEOIP', False):
#    urlpatterns += patterns(
#        '',
#        url(r'^map/$', display_map, name='tracking-visitor-map'),
#    )
